@@ -6,10 +6,10 @@
 **                          |/                                          **
 \*                                                                      */
 
-// $Id: Cell.scala 5880 2006-03-02 23:05:17Z mihaylov $
+// $Id: Cell.scala 8546 2006-08-31 13:52:34Z michelou $
 
 
-package scala;
+package scala
 
 
 /** A <code>Cell</code> is a generic wrapper which completely
@@ -19,4 +19,4 @@ package scala;
  *  @author  Martin Odersky
  *  @version 1.0, 08/08/2003
  */
-case class Cell[+T](elem: T);
+case class Cell[+T](elem: T)
