@@ -6,20 +6,21 @@
 **                          |/                                          **
 \*                                                                      */
 
-// $Id: Reaction.scala 9313 2006-11-21 15:07:53Z phaller $
+// $Id: Reaction.scala 9681 2007-01-23 14:12:33Z phaller $
 
 
 package scala.actors
 
 import java.lang.{InterruptedException, Runnable}
-import java.util.logging.{Level, Logger}
+
+class ExitActorException extends Throwable
 
 /**
  * The abstract class <code>Reaction</code> associates
  * an instance of an <code>Actor</code> with a
  * <code>java.lang.Runnable</code>.
  *
- * @version 0.9.0
+ * @version 0.9.2
  * @author Philipp Haller
  */
 private[actors] class Reaction(a: Actor,
@@ -29,72 +30,45 @@ private[actors] class Reaction(a: Actor,
 
   def actor = a
 
-  /**
-   * @param t ...
-   */
-  def log(t: Throwable): unit = {
-    Debug.info("logging "+t)
-    val logger = Logger.getLogger("Scheduler")
-    val buf = new StringBuffer
-    buf.append("Exception caught by task:\n")
-    buf.append(t.toString()+"\n")
-    val trace = t.getStackTrace()
-    for (val elem <- trace) {
-      buf.append(elem.toString() + "\n")
-    }
-    logger.log(Level.FINE, buf.toString())
-  }
-
   def run(): Unit = {
-    val t = currentThread
+    /*val t = currentThread
     val saved = Actor.selfs.get(t).asInstanceOf[Actor]
-    Actor.selfs.put(t, a)
+    Actor.selfs.put(t, a)*/
+    val saved = Actor.tl.get.asInstanceOf[Actor]
+    Actor.tl.set(a)
     Scheduler.unPendReaction
     a.isDetached = false
     try {
-      if (f == null)
-        a.act()
-      else
-        f(msg)
-
-      if (currentThread.isInterrupted())
-        throw new InterruptedException
-
-      a.kill()
-
-      if (currentThread.isInterrupted())
-        throw new InterruptedException
-
-      a.exit("normal")
-
-      if (currentThread.isInterrupted())
-        throw new InterruptedException
+      try {
+        // links
+        if (a.shouldExit)
+          a.exit()
+        else {
+          if (f == null)
+            a.act()
+          else
+            f(msg)
+          a.exit()
+        }
+      } catch {
+        case _: ExitActorException =>
+      }
     }
     catch {
-      case ie: InterruptedException => {
-        log(ie)
-        a.exitLinked()
-      }
-      case d: SuspendActorException => {
+      case _: SuspendActorException => {
         // do nothing (continuation is already saved)
       }
-      case t: Throwable => {
-        log(t)
-        a.exitLinked()
+      case _: Throwable => {
+        // links
+        if (!a.links.isEmpty) {
+          a.exitLinked()
+        }
       }
     }
     finally {
-      Actor.selfs.put(t, saved)
+      //Actor.selfs.put(t, saved)
+      Actor.tl.set(saved)
     }
   }
 
-  private var runnable = false
-
-  def isRunnable = synchronized {
-    runnable
-  }
-
-  def setRunnable(on: boolean) = synchronized {
-    runnable = on
-  }
 }
