@@ -6,7 +6,7 @@
 **                          |/                                          **
 \*                                                                      */
 
-// $Id: ArrayBuffer.scala 9215 2006-11-10 23:04:57Z mihaylov $
+// $Id: ArrayBuffer.scala 9669 2007-01-22 21:04:13Z odersky $
 
 
 package scala.collection.mutable
@@ -41,8 +41,18 @@ class ArrayBuffer[A] extends Buffer[A] with ResizableArray[A] {
    *  @param iter  the iterable object.
    *  @return      the updated buffer.
    */
-  override def ++(iter: Iterable[A]): Buffer[A] = {
-    insertAll(size, iter); this
+  override def ++=(iter: Iterable[A]): Unit = iter copyToBuffer this
+
+  /** Appends a number of elements in an array
+   *
+   *  @param src    the array
+   *  @param start  the first element to append
+   *  @param len    the number of elements to append           
+   */
+  override def ++=(src: Array[A], start: int, len: int): Unit = {
+    ensureSize(size + len)
+    Array.copy(src, start, array, size, len)
+    size = size + len
   }
     
   /** Prepends a single element to this buffer and return
@@ -134,7 +144,7 @@ class ArrayBuffer[A] extends Buffer[A] with ResizableArray[A] {
   
   /** Clears the buffer contents.
    */
-  def clear: Unit = {
+  def clear(): Unit = {
     size = 0
   }
   
