@@ -1,15 +1,20 @@
 /*                     __                                               *\
 **     ________ ___   / /  ___     Scala API                            **
-**    / __/ __// _ | / /  / _ |    (c) 2002-2006, LAMP/EPFL             **
+**    / __/ __// _ | / /  / _ |    (c) 2002-2007, LAMP/EPFL             **
 **  __\ \/ /__/ __ |/ /__/ __ |                                         **
 ** /____/\___/_/ |_/____/_/ | |                                         **
 **                          |/                                          **
 \*                                                                      */
 
-// $Id: remote.scala 5880 2006-03-02 23:05:17Z mihaylov $
+// $Id: remote.scala 9400 2006-11-28 17:22:45Z michelou $
 
 
-package scala;
-
+package scala
 
+/**
+ * An attribute that designates the class to which it is applied as remotable.
+ *
+ * @see Method <a href="ScalaObject.html#$tag()">$tag</a> in trait
+ *      <a href="ScalaObject.html">scala.ScalaObject</a>.
+ */
 class remote extends Attribute {}
