@@ -6,7 +6,7 @@
 **                          |/                                          **
 \*                                                                      */
 
-// $Id: BoxedAnyArray.scala 9110 2006-11-01 16:03:28Z mihaylov $
+// $Id: BoxedAnyArray.scala 9546 2007-01-03 15:56:13Z odersky $
 
 
 package scala.runtime
@@ -27,48 +27,49 @@ final class BoxedAnyArray(val length: Int) extends BoxedArray {
   private var unboxed: AnyRef = null
   private var elemClass: Class = null
 
-  def apply(index: Int): AnyRef = synchronized {
-    if (unboxed == null)
+  def apply(index: Int): Any = synchronized {
+    if (unboxed eq null)
       boxed(index);
-    else if (elemClass eq ScalaRunTime.IntTYPE) 
-      BoxedInt.box(unboxed.asInstanceOf[Array[Int]](index))
-    else if (elemClass eq ScalaRunTime.DoubleTYPE) 
-      BoxedDouble.box(unboxed.asInstanceOf[Array[Double]](index))
-    else if (elemClass eq ScalaRunTime.FloatTYPE) 
-      BoxedFloat.box(unboxed.asInstanceOf[Array[Float]](index))
-    else if (elemClass eq ScalaRunTime.LongTYPE) 
-      BoxedLong.box(unboxed.asInstanceOf[Array[Long]](index))
-    else if (elemClass eq ScalaRunTime.CharTYPE) 
-      BoxedChar.box(unboxed.asInstanceOf[Array[Char]](index))
-    else if (elemClass eq ScalaRunTime.ByteTYPE) 
-      BoxedByte.box(unboxed.asInstanceOf[Array[Byte]](index))
-    else if (elemClass eq ScalaRunTime.ShortTYPE) 
-      BoxedShort.box(unboxed.asInstanceOf[Array[Short]](index))
-    else if (elemClass eq ScalaRunTime.BooleanTYPE) 
-      BoxedBoolean.box(unboxed.asInstanceOf[Array[Boolean]](index))
+    else if (elemClass eq ScalaRunTime.IntTYPE)
+      Int.box(unboxed.asInstanceOf[Array[Int]](index))
+    else if (elemClass eq ScalaRunTime.DoubleTYPE)
+      Double.box(unboxed.asInstanceOf[Array[Double]](index))
+    else if (elemClass eq ScalaRunTime.FloatTYPE)
+      Float.box(unboxed.asInstanceOf[Array[Float]](index))
+    else if (elemClass eq ScalaRunTime.LongTYPE)
+      Long.box(unboxed.asInstanceOf[Array[Long]](index))
+    else if (elemClass eq ScalaRunTime.CharTYPE)
+      Char.box(unboxed.asInstanceOf[Array[Char]](index))
+    else if (elemClass eq ScalaRunTime.ByteTYPE)
+      Byte.box(unboxed.asInstanceOf[Array[Byte]](index))
+    else if (elemClass eq ScalaRunTime.ShortTYPE)
+      Short.box(unboxed.asInstanceOf[Array[Short]](index))
+    else if (elemClass eq ScalaRunTime.BooleanTYPE)
+      Boolean.box(unboxed.asInstanceOf[Array[Boolean]](index))
     else
       unboxed.asInstanceOf[Array[AnyRef]](index)
   }
 
-  def update(index: Int, elem: AnyRef): Unit = synchronized {
-    if (unboxed == null)
+  def update(index: Int, _elem: Any): Unit = synchronized {
+    val elem = _elem.asInstanceOf[AnyRef]
+    if (unboxed eq null)
       boxed(index) = elem
-    else if (elemClass eq ScalaRunTime.IntTYPE) 
-      unboxed.asInstanceOf[Array[Int]](index) = elem.asInstanceOf[BoxedNumber].intValue()
-    else if (elemClass eq ScalaRunTime.DoubleTYPE) 
-      unboxed.asInstanceOf[Array[Double]](index) = elem.asInstanceOf[BoxedNumber].doubleValue()
-    else if (elemClass eq ScalaRunTime.FloatTYPE) 
-      unboxed.asInstanceOf[Array[Float]](index) = elem.asInstanceOf[BoxedNumber].floatValue()
-    else if (elemClass eq ScalaRunTime.LongTYPE) 
-      unboxed.asInstanceOf[Array[Long]](index) = elem.asInstanceOf[BoxedNumber].longValue()
-    else if (elemClass eq ScalaRunTime.CharTYPE) 
-      unboxed.asInstanceOf[Array[Char]](index) = elem.asInstanceOf[BoxedNumber].charValue()
-    else if (elemClass eq ScalaRunTime.ByteTYPE) 
-      unboxed.asInstanceOf[Array[Byte]](index) = elem.asInstanceOf[BoxedNumber].byteValue()
-    else if (elemClass eq ScalaRunTime.ShortTYPE) 
-      unboxed.asInstanceOf[Array[Short]](index) = elem.asInstanceOf[BoxedNumber].shortValue()
-    else if (elemClass eq ScalaRunTime.BooleanTYPE) 
-      unboxed.asInstanceOf[Array[Boolean]](index) = elem.asInstanceOf[BoxedBoolean].value
+    else if (elemClass eq ScalaRunTime.IntTYPE)
+      unboxed.asInstanceOf[Array[Int]](index) = Int.unbox(elem)
+    else if (elemClass eq ScalaRunTime.DoubleTYPE)
+      unboxed.asInstanceOf[Array[Double]](index) = Double.unbox(elem)
+    else if (elemClass eq ScalaRunTime.FloatTYPE)
+      unboxed.asInstanceOf[Array[Float]](index) = Float.unbox(elem)
+    else if (elemClass eq ScalaRunTime.LongTYPE)
+      unboxed.asInstanceOf[Array[Long]](index) = Long.unbox(elem)
+    else if (elemClass eq ScalaRunTime.CharTYPE)
+      unboxed.asInstanceOf[Array[Char]](index) = Char.unbox(elem)
+    else if (elemClass eq ScalaRunTime.ByteTYPE)
+      unboxed.asInstanceOf[Array[Byte]](index) = Byte.unbox(elem)
+    else if (elemClass eq ScalaRunTime.ShortTYPE)
+      unboxed.asInstanceOf[Array[Short]](index) = Short.unbox(elem)
+    else if (elemClass eq ScalaRunTime.BooleanTYPE)
+      unboxed.asInstanceOf[Array[Boolean]](index) = Boolean.unbox(elem)
     else
       unboxed.asInstanceOf[Array[AnyRef]](index) = elem
   }
@@ -85,14 +86,13 @@ final class BoxedAnyArray(val length: Int) extends BoxedArray {
     else unbox(Platform.getClassForName(elemTag))
 
   def unbox(elemClass: Class): AnyRef = synchronized {
-    if (unboxed == null) {
+    if (unboxed eq null) {
       this.elemClass = elemClass;
       if (elemClass eq ScalaRunTime.IntTYPE) {
 	val newvalue = new Array[Int](length)
 	var i = 0
 	while (i < length) {
-          val x = boxed(i).asInstanceOf[BoxedNumber]
-	  if (x ne null) newvalue(i) = x.intValue(); 
+	  newvalue(i) = Int.unbox(boxed(i))
 	  i = i + 1
 	}
 	unboxed = newvalue
@@ -100,8 +100,7 @@ final class BoxedAnyArray(val length: Int) extends BoxedArray {
 	val newvalue = new Array[Double](length)
 	var i = 0
 	while (i < length) {
-          val x = boxed(i).asInstanceOf[BoxedNumber];
-	  if (x ne null) newvalue(i) = x.doubleValue(); 
+	  newvalue(i) = Double.unbox(boxed(i))
 	  i = i + 1
 	}
 	unboxed = newvalue;
@@ -109,8 +108,7 @@ final class BoxedAnyArray(val length: Int) extends BoxedArray {
 	val newvalue = new Array[Float](length)
 	var i = 0
 	while (i < length) {
-          val x = boxed(i).asInstanceOf[BoxedNumber];
-	  if (x ne null) newvalue(i) = x.floatValue(); 
+	  newvalue(i) = Float.unbox(boxed(i))
 	  i = i + 1
 	}
 	unboxed = newvalue;
@@ -118,8 +116,7 @@ final class BoxedAnyArray(val length: Int) extends BoxedArray {
 	val newvalue = new Array[Long](length)
 	var i = 0
 	while (i < length) {
-          val x = boxed(i).asInstanceOf[BoxedNumber]
-	  if (x ne null) newvalue(i) = x.longValue(); 
+	  newvalue(i) = Long.unbox(boxed(i))
 	  i = i + 1
 	}
 	unboxed = newvalue;
@@ -127,8 +124,7 @@ final class BoxedAnyArray(val length: Int) extends BoxedArray {
 	val newvalue = new Array[Char](length)
 	var i = 0
 	while (i < length) {
-          val x = boxed(i).asInstanceOf[BoxedNumber]
-	  if (x ne null) newvalue(i) = x.charValue(); 
+	  newvalue(i) = Char.unbox(boxed(i))
 	  i = i + 1
 	}
 	unboxed = newvalue
@@ -136,8 +132,7 @@ final class BoxedAnyArray(val length: Int) extends BoxedArray {
 	val newvalue = new Array[Byte](length)
 	var i = 0
 	while (i < length) {
-          val x = boxed(i).asInstanceOf[BoxedNumber]
-	  if (x ne null) newvalue(i) = x.byteValue(); 
+	  newvalue(i) = Byte.unbox(boxed(i))
 	  i = i + 1
 	}
 	unboxed = newvalue;
@@ -145,8 +140,7 @@ final class BoxedAnyArray(val length: Int) extends BoxedArray {
 	val newvalue = new Array[Short](length)
 	var i = 0
 	while (i < length) {
-          val x = boxed(i).asInstanceOf[BoxedNumber]
-	  if (x ne null) newvalue(i) = x.shortValue(); 
+	  newvalue(i) = Short.unbox(boxed(i))
 	  i = i + 1
 	}
 	unboxed = newvalue;
@@ -154,8 +148,7 @@ final class BoxedAnyArray(val length: Int) extends BoxedArray {
 	val newvalue = new Array[Boolean](length)
 	var i = 0
 	while (i < length) {
-          val x = boxed(i).asInstanceOf[BoxedBoolean];
-	  if (x ne null) newvalue(i) = x.value; 
+	  newvalue(i) = Boolean.unbox(boxed(i))
 	  i = i + 1
 	}
 	unboxed = newvalue;
@@ -173,21 +166,21 @@ final class BoxedAnyArray(val length: Int) extends BoxedArray {
 
   override def equals(other: Any): Boolean = (
     other.isInstanceOf[BoxedAnyArray] && (this eq (other.asInstanceOf[BoxedAnyArray])) ||
-    (if (unboxed == null) boxed == other else unboxed == other)
+    (if (unboxed eq null) boxed == other else unboxed == other)
   )
 
   override def hashCode(): Int = hash
 
   def value: AnyRef = {
-    if (unboxed == null) throw new NotDefinedError("BoxedAnyArray.value")
+    if (unboxed eq null) throw new NotDefinedError("BoxedAnyArray.value")
     unboxed
   }
 
   private def adapt(other: AnyRef): AnyRef =
-    if (this.unboxed == null)
+    if (this.unboxed eq null)
       other match {
-        case that: BoxedAnyArray => 
-          if (that.unboxed == null) {
+        case that: BoxedAnyArray =>
+          if (that.unboxed eq null) {
             that.boxed
           } else {
             if (ScalaRunTime.isValueClass(that.elemClass)) unbox(that.elemClass); 
@@ -216,8 +209,8 @@ final class BoxedAnyArray(val length: Int) extends BoxedArray {
       }
     else
       other match {
-        case that: BoxedAnyArray => 
-          if (that.unboxed != null) that.unboxed
+        case that: BoxedAnyArray =>
+          if (that.unboxed ne null) that.unboxed
           else if (ScalaRunTime.isValueClass(this.elemClass)) that.unbox(this.elemClass)
           else that.boxed
         case that: BoxedArray =>
@@ -228,12 +221,12 @@ final class BoxedAnyArray(val length: Int) extends BoxedArray {
 
   override def copyFrom(src: AnyRef, from: Int, to: Int, len: Int): Unit = {
     val src1 = adapt(src)
-    Array.copy(src1, from, if (unboxed != null) unboxed else boxed, to, len)
+    Array.copy(src1, from, if (unboxed ne null) unboxed else boxed, to, len)
   }
 
   override def copyTo(from: Int, dest: AnyRef, to: Int, len: Int): Unit = {
     var dest1 = adapt(dest)
-    Array.copy(if (unboxed != null) unboxed else boxed, from, dest1, to, len)
+    Array.copy(if (unboxed ne null) unboxed else boxed, from, dest1, to, len)
   }
 
   override def subArray(start: Int, end: Int): AnyRef = {
@@ -242,7 +235,7 @@ final class BoxedAnyArray(val length: Int) extends BoxedArray {
     result
   }
 
-  override def filter(p: Any => Boolean): AnyRef = {
+  final override def filter(p: Any => Boolean): BoxedArray = {
     val include = new Array[Boolean](length)
     var len = 0
     var i = 0
