@@ -6,7 +6,7 @@
 **                          |/                                          **
 \*                                                                      */
 
-// $Id: BoxedBooleanArray.scala 9162 2006-11-06 14:33:04Z mihaylov $
+// $Id: BoxedBooleanArray.scala 9546 2007-01-03 15:56:13Z odersky $
 
 
 package scala.runtime
@@ -19,10 +19,10 @@ final class BoxedBooleanArray(val value: Array[Boolean]) extends BoxedArray {
 
   def length: Int = value.length
 
-  def apply(index: Int): AnyRef = Boolean.box(value(index))
+  def apply(index: Int): Any = Boolean.box(value(index))
 
-  def update(index: Int, elem: AnyRef): Unit = {
-    value(index) = Boolean.unbox(elem)
+  def update(index: Int, elem: Any): Unit = {
+    value(index) = Boolean.unbox(elem.asInstanceOf[AnyRef])
   }
 
   def unbox(elemTag: String): AnyRef = value
@@ -40,7 +40,7 @@ final class BoxedBooleanArray(val value: Array[Boolean]) extends BoxedArray {
     result
   }
 
-  def filter(p: Any => Boolean): Array[Boolean] = {
+  final override def filter(p: Any => Boolean): BoxedArray = {
     val include = new Array[Boolean](value.length)
     var len = 0
     var i = 0
@@ -55,6 +55,6 @@ final class BoxedBooleanArray(val value: Array[Boolean]) extends BoxedArray {
       if (include(i)) { result(len) = value(i); len = len + 1 }
       i = i + 1
     }
-    result
+    new BoxedBooleanArray(result)
   }
 }
