@@ -2,7 +2,7 @@
  * Copyright 2005-2006 LAMP/EPFL
  * @author  Martin Odersky
  */
-// $Id: CompilationUnits.scala 9324 2006-11-22 16:22:05Z odersky $
+// $Id: CompilationUnits.scala 9605 2007-01-14 19:44:59Z spoon $
 
 package scala.tools.nsc
 
@@ -12,6 +12,9 @@ import scala.collection.mutable.HashSet
 
 trait CompilationUnits requires Global {
 
+  /** One unit of compilation that has been submitted to the compiler.
+    * It typically corresponds to a single file of source code.  It includes
+    * error-reporting hooks.  */
   class CompilationUnit(val source: SourceFile) {
 
     /** the fresh name creator */
@@ -42,6 +45,12 @@ trait CompilationUnits requires Global {
       if (settings.unchecked.value) reporter.warning(position(pos), msg)
       else currentRun.uncheckedWarnings = true
 
+    def incompleteInputError(pos:int, msg:String) =
+      if (!(errorPositions contains pos)) {
+        errorPositions += pos
+        reporter.incompleteInputError(position(pos), msg)
+      }
+    
     override def toString() = source.toString()
 
     def clear() = {
