@@ -2,7 +2,7 @@
  * Copyright 2005-2006 LAMP/EPFL
  * @author  Martin Odersky
  */
-// $Id: CompileClient.scala 9235 2006-11-13 14:59:18Z mihaylov $
+// $Id: CompileClient.scala 9692 2007-01-23 18:35:37Z michelou $
 
 package scala.tools.nsc
 
@@ -18,12 +18,13 @@ import scala.tools.util.StringOps
 object CompileClient {
   val PRODUCT: String = getProperty("scala.tool.name", "scalac")
   val VERSION: String = getProperty("scala.tool.version", "unknown version")
-  val COPYRIGHT: String = getProperty("scala.copyright", "(c) 2002-2006 LAMP/EPFL")
+  val COPYRIGHT: String = getProperty("scala.copyright", "(c) 2002-2007 LAMP/EPFL")
 
   val versionMsg = PRODUCT + " " + VERSION + " -- " + COPYRIGHT
 
   var verbose = false
   var version = false
+  var shutdown = false
 
   /** Convert a filename to an absolute path */
   def absFileName(path: String) = new File(path).getAbsolutePath()
@@ -53,6 +54,8 @@ object CompileClient {
         verbose = true
       } else if (arg == "-version") {
         version = true
+      } else if (arg == "-shutdown") {
+        shutdown = true
       }
       i = i + 1
       if (i < args.length) {
@@ -90,8 +93,12 @@ object CompileClient {
       Console.println("[Server arguments: " + args.mkString("", " ", "]"))
       Console.println("[VM arguments: " + vmArgs + "]")
     }
-    val socket = if (serverAdr == "") CompileSocket.getOrCreateSocket(vmArgs)
+    val socket = if (serverAdr == "") CompileSocket.getOrCreateSocket(vmArgs, !shutdown)
                  else CompileSocket.getSocket(serverAdr)
+    if(shutdown && (socket==null)) {
+      Console.println("[No compilation server running.]")
+      return 0
+    }
     val out = new PrintWriter(socket.getOutputStream(), true)
     val in = new BufferedReader(new InputStreamReader(socket.getInputStream()))
     out.println(CompileSocket.getPassword(socket.getPort()))
