@@ -2,7 +2,7 @@
  * Copyright 2005-2006 LAMP/EPFL
  * @author  Martin Odersky
  */
-// $Id: Global.scala 9431 2006-11-30 14:20:55Z odersky $
+// $Id: Global.scala 9685 2007-01-23 15:34:40Z spoon $
 
 package scala.tools.nsc
 
@@ -35,6 +35,16 @@ class Global(var settings: Settings, var reporter: Reporter) extends SymbolTable
                                                              with Trees
                                                              with CompilationUnits
 {
+  // alternate constructors ------------------------------------------
+  def this(reporter: Reporter) =
+    this(new Settings(err => reporter.error(null,err)), 
+         reporter)
+         
+  def this(settings: Settings) =
+    this(settings, new ConsoleReporter)
+         
+  def this() = this(new Settings, new ConsoleReporter)
+  
   // sub-components --------------------------------------------------
   object treePrinters extends TreePrinters {
     val global: Global.this.type = Global.this
