@@ -2,7 +2,7 @@
  * Copyright 2005-2006 LAMP/EPFL
  * @author  Martin Odersky
  */
-// $Id: Interpreter.scala 9431 2006-11-30 14:20:55Z odersky $
+// $Id: Interpreter.scala 9687 2007-01-23 15:54:05Z michelou $
 
 package scala.tools.nsc
 
@@ -18,6 +18,7 @@ import io.PlainFile
 import reporters.{ConsoleReporter, Reporter}
 import symtab.Flags
 import util.SourceFile
+import nsc.{InterpreterResults=>IR}
 
 /** <p>
  *    An interpreter for Scala code.
@@ -80,6 +81,17 @@ class Interpreter(val settings: Settings, reporter: Reporter, out: PrintWriter)
   /** Be quiet.  Do not print out the results of each
     * submitted command unless an exception is thrown.  */
   def beQuiet = { printResults = false }
+  
+  /** Temporarily be quiet */
+  def beQuietDuring[T](operation: => T): T = {
+    val wasPrinting = printResults
+    try {
+      printResults = false
+      operation
+    } finally {
+      printResults = wasPrinting
+    }
+  }
 
   /** directory to save .class files to */
   val classfilePath = File.createTempFile("scalaint", "")
@@ -119,7 +131,7 @@ class Interpreter(val settings: Settings, reporter: Reporter, out: PrintWriter)
                           parentClassLoader)
 
   /** XXX Let's get rid of this.  I believe the Eclipse plugin is
-    * the only user of it.   */
+    * the only user of it, so this should be doable.  */
   protected def parentClassLoader: ClassLoader = null
 
   /** the previous requests this interpreter has processed */
@@ -136,7 +148,8 @@ class Interpreter(val settings: Settings, reporter: Reporter, out: PrintWriter)
   private def newLineName = {
     val num = nextLineNo
     nextLineNo = nextLineNo + 1
-    compiler.nme.INTERPRETER_LINE_PREFIX + num
+    (compiler.nme.INTERPRETER_LINE_PREFIX +
+      (if (num>=0) num.toString else "_neg" + (-num).toString))
   }
 
   /** import statements that should be used for submitted code */
@@ -161,39 +174,37 @@ class Interpreter(val settings: Settings, reporter: Reporter, out: PrintWriter)
     stringWriter.toString
   }
 
-  /** Parse a line into a sequence of trees.
-   *
-   *  @param line ...
-   *  @return     ...
-   */
-  private def parse(line: String): List[Tree] = {
-    // simple parse: just parse it, nothing else
-    def simpleParse(code: String): List[Tree] = {
-      val unit =
-        new CompilationUnit(
-          new SourceFile("<console>", code.toCharArray()))
+  /** Parse a line into a sequence of trees. Returns None if the input
+    * is incomplete. */
+  private def parse(line: String): Option[List[Tree]] = {
+    var justNeedsMore = false
+    reporter.withIncompleteHandler((pos,msg) => {justNeedsMore = true}) {
+      // simple parse: just parse it, nothing else
+      def simpleParse(code: String): List[Tree] = {
+        val unit =
+          new CompilationUnit(
+            new SourceFile("<console>", code.toCharArray()))
+        new compiler.syntaxAnalyzer.Parser(unit).templateStatSeq
+      }
 
-      new compiler.syntaxAnalyzer.Parser(unit).templateStatSeq
-    }
-
-    // parse the main code along with the imports
-    reporter.reset
-    val trees = simpleParse(codeForImports + line)
-    if (reporter.hasErrors)
-      Nil // the result did not parse, so stop
-    else {
-      // parse the imports alone
-      val importTrees = simpleParse(codeForImports)
-
-      // return just the new trees, not the import trees
-      trees.drop(importTrees.length)
+      // parse the main code along with the imports
+      reporter.reset
+      val trees = simpleParse(codeForImports + line)
+      if (justNeedsMore)
+        None
+      else if (reporter.hasErrors)
+        Some(Nil) // the result did not parse, so stop
+      else {
+        // parse the imports alone
+        val importTrees = simpleParse(codeForImports)
+  
+        // return just the new trees, not the import trees
+        Some(trees.drop(importTrees.length))
+      }
     }
   }
 
-  /** Compile one source file.
-   *
-   *  @param filename
-   */
+  /** Compile one source file. */
   def compileFile(filename: String): Unit = {
     val jfile = new File(filename)
     if (!jfile.exists) {
@@ -207,9 +218,6 @@ class Interpreter(val settings: Settings, reporter: Reporter, out: PrintWriter)
 
   /** Compile an nsc SourceFile.  Returns true if there are
    *  no compilation errors, or false othrewise.
-   *
-   *  @param sources ...
-   *  @return        ...
    */
   def compileSources(sources: List[SourceFile]): Boolean = {
     val cr = new compiler.Run
@@ -220,19 +228,17 @@ class Interpreter(val settings: Settings, reporter: Reporter, out: PrintWriter)
 
   /** Compile a string.  Returns true if there are no
    *  compilation errors, or false otherwise.
-   *
-   *  @param code ...
-   *  @return     ...
    */
   def compileString(code: String): Boolean =
     compileSources(List(new SourceFile("<script>", code.toCharArray)))
 
-  /** build a request from the user.  "tree" is "line" after being parsed.
+  /** Build a request from the user. <code>trees</code> is <code>line</code>
+   *  after being parsed.
    *
-   *  @param trees    ...
-   *  @param line     ...
-   *  @param lineName ...
-   *  @return         ...
+   *  @param trees    ..
+   *  @param line     ..
+   *  @param lineName ..
+   *  @return         ..
    */
   private def buildRequest(trees: List[Tree], line: String, lineName: String): Request =
     trees match {
@@ -252,7 +258,6 @@ class Interpreter(val settings: Settings, reporter: Reporter, out: PrintWriter)
       case List(_:AliasTypeDef) => new TypeAliasReq(line, lineName)
       case List(_:Import) => new ImportReq(line, lineName)
       case _ => {
-        //reporter.error(null, trees.toString)
         reporter.error(null, "That kind of statement combination is not supported by the interpreter.")
         null
       }
@@ -272,19 +277,22 @@ class Interpreter(val settings: Settings, reporter: Reporter, out: PrintWriter)
    *  @param line ...
    *  @return     ...
    */
-  def interpret(line: String): boolean = {
+  def interpret(line: String): IR.Result = {
     // parse
-    val trees = parse(line)
-    if (trees.isEmpty) return false  // parse error or empty input
+    val trees = parse(line) match {
+      case None => return IR.Incomplete
+      case Some(Nil) => return IR.Error // parse error or empty input
+      case Some(trees) => trees
+    }
 
     val lineName = newLineName
 
     // figure out what kind of request
     val req = buildRequest(trees, line, lineName)
-    if (req eq null) return false  // a disallowed statement type
+    if (req eq null) return IR.Error  // a disallowed statement type
 
     if (!req.compile)
-      return false  // an error happened during compilation, e.g. a type error
+      return IR.Error  // an error happened during compilation, e.g. a type error
 
     val Pair(interpreterResultString, succeeded) = req.loadAndRun
 
@@ -301,7 +309,7 @@ class Interpreter(val settings: Settings, reporter: Reporter, out: PrintWriter)
     if (succeeded)
       prevRequests += req
 
-    succeeded
+    if (succeeded) IR.Success else IR.Error
   }
 
   /** A counter used for numbering objects created by bind() */
@@ -341,6 +349,18 @@ class Interpreter(val settings: Settings, reporter: Reporter, out: PrintWriter)
   }
 
 
+  /** Make a dry run in order to fill caches.  This is useful
+    * for interactive interpreters so that the interpreter responds
+    * quickly to the first user-supplied query.
+    */
+  def prime: Unit = 
+    if(prevRequests.isEmpty)
+      beQuietDuring {
+        nextLineNo = -1  // cosmetic: make the first user-requested line
+                         // be number 0
+        interpret("0")
+      }
+
   /** <p>
    *    This instance is no longer needed, so release any resources
    *    it is using.
@@ -355,8 +375,8 @@ class Interpreter(val settings: Settings, reporter: Reporter, out: PrintWriter)
     Interpreter.deleteRecursively(classfilePath)
 
   /** A traverser that finds all mentioned identifiers, i.e. things
-      that need to be imported.
-      It might return extra names.  */
+   *  that need to be imported. It might return extra names.
+   */
   private class ImportVarsTraverser(definedVars: List[Name]) extends Traverser {
     val importVars = new HashSet[Name]()
 
@@ -368,7 +388,7 @@ class Interpreter(val settings: Settings, reporter: Reporter, out: PrintWriter)
 
   /** One line of code submitted by the user for interpretation */
   private abstract class Request(val line: String, val lineName: String) {
-    val trees = parse(line)
+    val Some(trees) = parse(line)
 
     /** name to use for the object that will compute "line" */
     def objectName = lineName + compiler.nme.INTERPRETER_WRAPPER_SUFFIX  // make it unlikely to clash with user variables
@@ -659,8 +679,8 @@ object Interpreter {
    */
   def deleteRecursively(path: File): Unit = {
     path match  {
-      case _ if (!path.exists) => ()
-      case _ if (path.isDirectory) =>
+      case _ if !path.exists => {}
+      case _ if path.isDirectory =>
         for (val p <- path.listFiles)
           deleteRecursively(p)
         path.delete
