@@ -2,16 +2,18 @@
  * Copyright 2005-2006 LAMP/EPFL
  * @author Alexander Spoon
  */
-// $Id: InterpreterLoop.scala 9110 2006-11-01 16:03:28Z mihaylov $
+// $Id: InterpreterLoop.scala 9687 2007-01-23 15:54:05Z michelou $
 
 package scala.tools.nsc
 
 import java.lang.System
 import java.lang.ClassLoader
-import java.io._
+import java.io.{BufferedReader, InputStreamReader, File, FileReader, PrintWriter}
+import java.io.IOException
 
 import scala.tools.nsc.reporters.{Reporter, ConsoleReporter}
 import scala.tools.nsc.util.{Position}
+import nsc.{InterpreterResults=>IR}
 
 /** The main loop of the command-line interface to the
  *  <a href="http://scala.epfl.ch/" target="_top">Scala</a> interpreter.
@@ -20,10 +22,18 @@ import scala.tools.nsc.util.{Position}
  *  @author  Lex Spoon
  *  @version 1.0
  */
-class InterpreterLoop(in: BufferedReader, out: PrintWriter) {
+class InterpreterLoop(in0: BufferedReader, out: PrintWriter) {
   def this() = this(new BufferedReader(new InputStreamReader(System.in)),
                     new PrintWriter(System.out))
 
+  /** The input stream from which interpreter commands come */
+  var in = in0
+
+  /** Whether the interpreter is interactive (like normal), or instead
+    * is reading from a file.
+    */
+  var interactive = true
+  
   var settings: Settings = _ // set by main()
   var interpreter: Interpreter = null // set by createInterpreter()
 
@@ -41,7 +51,7 @@ class InterpreterLoop(in: BufferedReader, out: PrintWriter) {
   /** Close the interpreter, if there is one, and set
     * interpreter to null. */
   def closeInterpreter =
-    if (interpreter != null) {
+    if (interpreter ne null) {
       interpreter.close
       interpreter = null
     }
@@ -63,59 +73,80 @@ class InterpreterLoop(in: BufferedReader, out: PrintWriter) {
   }
 
   /** print a friendly help message */
-  def printHelp = {
+  def printHelp {
     out.println("This is an interpreter for Scala.")
     out.println("Type in expressions to have them evaluated.")
-    out.println("Type :quit to exit the interpreter.")
     out.println("Type :compile followed by a filename to compile a complete Scala file.")
-    out.println("Type :load followed by a filename to load a sequence of interpreter commands.")
+    out.println("Type :help to repeat this message.")
+    out.println("Type :load followed by a filename to load interpreter commands.")
     out.println("Type :replay to reset execution and replay all previous commands.")
-    out.println("Type :help to repeat this message later.")
+    out.println("Type :quit to exit the interpreter.")
+  }
+  
+  /** Print a welcome message */
+  def printWelcome {
+    out.println("This is an interpreter for Scala.")
+    out.println("Type in expressions to have them evaluated.")
+    out.println("Type :help for more information.")
   }
 
   /** The main read-eval-print loop for the interpereter.  It calls
    *  <code>command()</code> for each line of input, and stops when
-   *  <code>command()</code> returns false.
+   *  <code>command()</code> returns <code>false</code>.
    */
-  def repl(): Unit =
-    while(true) {
-      out.print("\nscala> ")
-      out.flush
+  def repl {
+    var firstTime = true
+    while (true) {
+      if (interactive) {
+        out.print("\nscala> ")
+        out.flush
+        if (firstTime) {
+          interpreter.prime
+          firstTime = false
+        }
+      }
       var line = in.readLine()
-      if (line == null)
+      if (line eq null)
         return ()  // assumes null means EOF
 
-      val Pair(keepGoing, shouldReplay) = command(line)
+      val Pair(keepGoing, finalLineMaybe) = command(line)
 
       if (!keepGoing)
-        return ()
-      if (shouldReplay)
-        addReplay(line)
+        return
+        
+      finalLineMaybe match {
+        case Some(finalLine) => addReplay(finalLine)
+        case None => ()
+      }
     }
+  }
 
   /** interpret all lines from a specified file */
-  def interpretAllFrom(filename: String): Unit = {
+  def interpretAllFrom(filename: String) {
     val fileIn = try {
       new FileReader(filename)
     } catch {
       case _:IOException =>
         out.println("Error opening file: " + filename)
-        null
+        return
     }
-    if (fileIn == null) return ()
-    val in = new BufferedReader(fileIn)
-    while (true) {
-      val line = in.readLine
-      if (line == null) {
-        fileIn.close
-        return ()
-      }
-      command(line)
+    val oldIn = in
+    val oldInteractive = interactive
+    try {
+      in = new BufferedReader(fileIn)
+      interactive = false
+      out.println("Loading " + filename + "...")
+      out.flush
+      repl
+    } finally {
+      in = oldIn
+      interactive = oldInteractive
+      fileIn.close
     }
   }
 
   /** create a new interpreter and replay all commands so far */
-  def replay = {
+  def replay {
     closeInterpreter
     createInterpreter
     for (val cmd <- replayCommands) {
@@ -125,10 +156,10 @@ class InterpreterLoop(in: BufferedReader, out: PrintWriter) {
     }
   }
 
-  /** Run one command submitted by the user.  Two values are returned:
-    * (1) whether to keep running, and (2) whether to record the
-    * command for replay. */
-  def command(line: String): Pair[Boolean, Boolean] = {
+  /** Run one command submitted by the user.  Three values are returned:
+    * (1) whether to keep running, (2) the line to record for replay,
+    * if any. */
+  def command(line: String): Pair[Boolean, Option[String]] = {
     def withFile(command: String)(action: String => Unit): Unit = {
       val spaceIdx = command.indexOf(' ')
       if (spaceIdx <= 0) {
@@ -149,35 +180,64 @@ class InterpreterLoop(in: BufferedReader, out: PrintWriter) {
     val loadRegexp    = ":l(o(a(d)?)?)?.*"
     val replayRegexp  = ":r(e(p(l(a(y)?)?)?)?)?.*"
 
-    var shouldReplay = false
+    var shouldReplay: Option[String] = None
 
     if (line.matches(helpRegexp))
       printHelp
     else if (line.matches(quitRegexp))
-      return Pair(false, false)
+      return Pair(false, None)
     else if (line.matches(compileRegexp)) {
       withFile(line)(f => {
         interpreter.compileFile(f)
-        shouldReplay = true
+        shouldReplay = Some(line)
       })
     }
     else if (line.matches(loadRegexp)) {
       withFile(line)(f => {
         interpretAllFrom(f)
-        shouldReplay = true
+        shouldReplay = Some(line)
       })
     }
     else if (line.matches(replayRegexp))
       replay
     else if (line.startsWith(":"))
       out.println("Unknown command.  Type :help for help.")
-    else {
-      if (interpreter.interpret(line))
-        shouldReplay = true
-    }
+    else 
+      shouldReplay = interpretStartingWith(line)
+
     Pair(true, shouldReplay)
   }
+  
+  /** Interpret expressions starting with the first line.
+    * Read lines until a complete compilation unit is available
+    * or until a syntax error has been seen.  If a full unit is
+    * read, go ahead and interpret it.  Return the full string
+    * to be recorded for replay, if any.
+    */
+  def interpretStartingWith(code: String): Option[String] =
+  {
+    interpreter.interpret(code) match {
+      case IR.Success => Some(code)
+      case IR.Error => None
+      case IR.Incomplete =>
+        if (interactive && code.endsWith("\n\n")) {
+          out.println("Two blank lines seen.  Aborting this expression.")
+          None
+        } else {
+          if (interactive) {
+            out.print("     | ")
+            out.flush
+          }
+          val nextLine = in.readLine
+          if (nextLine == null)
+            None  // end of file
+          else
+            interpretStartingWith(code + "\n" + nextLine)
+        }
+    }
+  }
 
+      
   def main(settings: Settings) = {
     this.settings = settings
     
@@ -190,7 +250,7 @@ class InterpreterLoop(in: BufferedReader, out: PrintWriter) {
     createInterpreter
 
     try {
-      printHelp
+      printWelcome
       repl
     } finally {
       closeInterpreter
@@ -198,7 +258,7 @@ class InterpreterLoop(in: BufferedReader, out: PrintWriter) {
   }
   
   /** process command-line arguments and do as they request */
-  def main(args: Array[String]): unit = {
+  def main(args: Array[String]) {
     def error1(msg: String): Unit = out.println("scala: " + msg)
     val command = new InterpreterCommand(List.fromArray(args), error1)
     
@@ -207,9 +267,8 @@ class InterpreterLoop(in: BufferedReader, out: PrintWriter) {
       // explicitly requested a help listing
       out.println(command.usageMsg)
       out.flush
-      return ()
     }
-
-    main(command.settings)
+    else
+      main(command.settings)
   }
 }
